
/**
 * @imports
 */
import _isClass from '@web-native-js/commons/js/isClass.js';
import _isFunction from '@web-native-js/commons/js/isFunction.js';

/**
 * ---------------------------
 * The Registry class
 * ---------------------------
 *
 * This class provides window-like functionality as a component.
 */
			
export default {
	
	/**
	 * Finds and instantiates a registered component.
	 *
	 * @param string				name
	 * @param array					...args
	 *
	 * @return Component
	 */
	get(name, ...args) {
		var construtor, instance;
		if (construtor = registry[name]) {
			if (construtor.singletonInstance) {
				return construtor.singletonInstance;
			}
			if (_isClass(construtor.component)) {
				instance = new construtor.component(...args);
			} else if (_isFunction(construtor.component)) {
				instance = construtor.component(...args);
			} else {
				instance = construtor.component;
			}
			if (construtor.singleton) {
				construtor.singletonInstance = instance;
			}
			return instance;
		}
		throw new Error('The "' + name + '" component is not defined in registry!');
	},
	
	/**
	 * Registers a logical component.
	 *
	 * @param string				name
	 * @param Component|function	component
	 * @param bool					singleton
	 *
	 * @return void
	 */
	set(name, component, singleton = false) {
		registry[name] = {component, singleton};
	},
};

/**
 * @param object
 */
const registry = {};
