
/**
 * @imports
 */
import {Reflex} from '@web-native-js/chtml';
import _each from '@web-native-js/commons/obj/each.js';
import _isFunction from '@web-native-js/commons/js/isFunction.js';
import _isClass from '@web-native-js/commons/js/isClass.js';
import _isNumeric from '@web-native-js/commons/js/isNumeric.js';
import _mixin from '@web-native-js/commons/js/mixin.js';
import Registry from '../../Registry.js';
import Collection from '../Collection.js';
import Item from '../Item.js';

/**
 * ---------------------------
 * The Route class
 * ---------------------------
 *
 * This class represents a given route and manages
 * its sub-route.
 */
			
export default class Route extends _mixin(Item, Collection) {

	/**
	 * Constructs a new Route instance.
	 * Sub-views may also be listed.
	 *
	 * Its content could either be static or remote.
	 *
	 * @param object				state
	 * @param object				params
	 *
	 * @return void
	 */
	constructor(items = {}, params = {}) {
		params.itemsOffset = 'subroutes';
		if (items.subroutes) {
			items.subroutes = Route.createRoutes(items.subroutes);
		}
		super(items, params);
		// -----------------------
		// Observe the Route's route-slot
		var routerInstance = Registry.get('router');
		var subroutingKey = this.subroutingKey || 0;
		// -----------------------
		Reflex.observe(routerInstance, _isNumeric(subroutingKey) ? 'pathsplit' : 'pathmap', path => {
			var subroutingKeyVal = path[subroutingKey];
			if (this.active && this.subroutes && subroutingKeyVal) {
				if (this.subroutes[subroutingKeyVal]) {
					return this.subroutes[subroutingKeyVal].setActiveState(true);
				} else if ('subroutingKey' in this) {
					throw new Error('404: ' + subroutingKeyVal, this.subroutingKey);
				}
			} else if (this.activeChild) {
				return this.activeChild.setActiveState(false);
			}
		});
		// -----------------------
		Reflex.observe(this, 'active', activeState => {
			if (activeState === true && this.activeCallback) {
				this.activeCallback();
			}
		});
		// -----------------------
		if (this.initCallback) {
			this.initCallback();
		}
	}

	/**
	 * Creates Routes from declarations.
	 *
	 * @param array			routesDeclaration
	 *
	 * @return Routeset
	 */
	static createRoutes(routesDeclaration) {
		var routes = {};
		_each(routesDeclaration, (name, routeConstructor) => {
			var route = routeConstructor instanceof Route ? routeConstructor
				: (_isClass(routeConstructor) ? new routeConstructor 
					: (_isFunction(routeConstructor) ? routeConstructor(match) 
						: new Route(routeConstructor || {})));
			Reflex.set(routes, name, route);
		});
		return routes;
	}
};