
/**
 * @imports
 */
import Reflex from '@web-native-js/reflex';
import Registry from '../../Registry.js';

/**
 * ---------------------------
 * A base class for Observer implementations.
 * ---------------------------
 */

export default class {
	
	/**
	 * Accepts elements (targets) and options for,
	 * and creates the, Observer instance.
	 *
	 * @param HTMLElement|DOMNodeList|array`	targets
	 * @param object 							options
	 *
	 * @return void
	 */
	constructor(targets = [], params = {}) {
		this.$ = {};
		this.$.targets = targets;
		this.$.params = params;
		this.$.customCallbacks = [];
	}
	
	/**
	 * @aliasof Reflex.observe().
	 *
	 * @param array 	...args
	 */
	observe(...args) {
		return Reflex.observe(this, ...args);
	}
	
	/**
	 * @aliasof Reflex.unobserve().
	 *
	 * @param array 	...args
	 */
	unobserve(...args) {
		return Reflex.unobserve(this, ...args);
	}
	
	/**
	 * Adda a new element (target) to the Observer.
	 * Optionally accepts a callback for handling the specific target.
	 *
	 * @param HTMLElement 	el
	 * @param function	`	callback
	 *
	 * @return void
	 */
	add(el, callback = null) {
		this.$.observer.observe(el);
		if (callback) {
			this.$.customCallbacks.push({el:el, callback:callback});
		}
	}
	
	/**
	 * (Internal) Publishes entries to state.
	 *
	 * @param IntersectionObserverEntry	`	entry
	 *
	 * @return void
	 */
	_publish(entry) {
		Reflex.set(this, 'entry', entry);
		var customCallback;
		if (customCallback = this.$.customCallbacks.filter(obj => obj.el === entry.target)[0]) {
			customCallback.callback(entry);
		}
	}
};
