
/**
 * @imports
 */
import _Chtml from '@web-native-js/chtml';
import {Reflex} from '@web-native-js/chtml';
import _remove from '@web-native-js/commons/arr/remove.js';
import _isUndefined from '@web-native-js/commons/js/isUndefined.js';
// -------------------------------------
import domFrom from './dom/from.js';
import domFind from './dom/find.js';
import domData from './dom/data.js';
import domAttr from './dom/attr.js';
import domAttrAsync from './dom/attrAsync.js';
import domHtml from './dom/html.js';
import domHtmlAsync from './dom/htmlAsync.js';
import domText from './dom/text.js';
import domTextAsync from './dom/textAsync.js';
import domAppend from './dom/append.js';
import domAppendAsync from './dom/appendAsync.js';
import domPrepend from './dom/prepend.js';
import domPrependAsync from './dom/prependAsync.js';
// -------------------------------------
import css from './css/css.js';
import cssAsync from './css/cssAsync.js';
import cssComputeAsync from './css/computeAsync.js';
import cssTransaction from './css/transaction.js';
// -------------------------------------
import uiTranslateTo from './ui/translateTo.js';
import uiOffsetTo from './ui/offsetTo.js';
import uiScrollTo from './ui/scrollTo.js';
// -------------------------------------
import Anim from './anim/Anim.js';
import Reflow from './Reflow.js';
import Registry from '../Registry.js';
import globalParams from '../params.js';
import on from './evt/on.js';
import off from './evt/off.js';
import trigger from './evt/trigger.js';

/**
 * ---------------------------
 * Chtml library.
 * ---------------------------
 */
export default class Chtml extends _Chtml {
	
	/**
	 * Overrides the Static paramter in super.from() with self.
	 *
	 * @see super.from()
	 */
	static from(input, params = {}, Static = Chtml) {
		return super.from(input, params, Static);
	}
	
	/**
	 * Adds a component to the instance;
	 * saves it at an automatically-derived name.
	 *
	 * @param string 		name	
	 * @param array 		...args	
	 *
	 * @inheritdoc
	 */
	use(name, ...args) {
		return this.useAs(name, '@' + name, ...args);
	}
	
	/**
	 * Adds a component to the instance;
	 * saves it at the given key.
	 *
	 * @param string 		name	
	 * @param string 		key	
	 * @param array 		...args	
	 *
	 * @return object
	 */
	useAs(name, key, ...args) {
		var componentInstance = Registry.get(name, ...args);
		Reflex.set(this, key, componentInstance, true/*returnEvent*/);
		return componentInstance;
	}

	// ----------------------------------------
	
	/**
	 * Creates an instance over an element.
	 *
	 * @param HTMLElement			el
	 * @param object				params
	 *
	 * @return this
	 */
	constructor(el, params) {
		super(el, params);
	}
	
	/**
	 * Strategically returns a HTMLElement.
	 *
	 * @return HTMLElement
	 */
	__el() {
		return this.el || document.createElement('div');
	}
	
	/**
	 * Automatically returns the current instance in place
	 * of the element.
	 *
	 * @param mixed					ret
	 *
	 * @return mixed
	 */
	__ret(ret) {
		return this.el && ret === this.el ? this : ret;
	}

	// ----------------------------------------
	
	/**
	 * Queries the current element.
	 *
	 * @param string 				selector
	 *
	 * @return DOMNodeList
	 */
	find(selector) {
		return new Chtml(this.el ? domFind(selector, this.el) : undefined);
	}

	// ----------------------------------------
	
	/**
	 * @see domData
	 *
	 * @return mixed|this
	 */
	data(requestOrPayload, val = null) {
		// The ...arguments is important as the
		// domData function is sensitive to argument count
		return this.__ret(domData(this.__el(), ...arguments));
	}

	// ----------------------------------------
	
	/**
	 * @see css
	 *
	 * @return mixed
	 */
	cssSync(...args) {
		// The ...arguments is important as the
		// css function is sensitive to argument count
		return this.__ret(css(this.__el(), ...arguments));
	}
	
	/**
	 * @see this.attrSync()
	 *
	 * @return mixed
	 */
	classSync(...args) {
		// The ...arguments is important as the
		// this.attrSync() function is sensitive to argument count
		return this.attrSync('class', ...arguments);
	}
	
	/**
	 * @see domAttr
	 *
	 * @return string|this
	 */
	attrSync(requestOrPayload, valOrMutation = null, subValMutation = null) {
		// The ...arguments is important as the
		// domAttr function is sensitive to argument count
		return this.__ret(domAttr(this.__el(), ...arguments));
	}
	
	/**
	 * @see domHtml
	 *
	 * @return string|this
	 */
	htmlSync(content = null) {
		// The ...arguments is important as the
		// domHtml function is sensitive to argument count
		return this.__ret(domHtml(this.__el(), ...arguments));
	}
	
	/**
	 * @see domText
	 *
	 * @return string|this
	 */
	textSync(content = null) {
		// The ...arguments is important as the
		// domText function is sensitive to argument count
		return this.__ret(domText(this.__el(), ...arguments));
	}
	
	/**
	 * @see domAppend
	 *
	 * @return string|this
	 */
	appendSync(content) {
		return this.__ret(domAppend(this.__el(), content));
	}
	
	/**
	 * @see domPrepend
	 *
	 * @return string|this
	 */
	prependSync(content) {
		return this.__ret(domPrepend(this.__el(), content));
	}

	// ----------------------------------------
	
	/**
	 * @see cssAsync
	 *
	 * @return Promise
	 */
	css(...args) {
		// The ...arguments is important as the
		// cssAsync function is sensitive to argument count
		return cssAsync(this.__el(), ...arguments).then(ret => {
			return this.el && ret === this.el ? this : ret;
		});
	}
	
	/**
	 * @see this.attr()
	 *
	 * @return mixed
	 */
	class(...args) {
		// The ...arguments is important as the
		// this.attr() function is sensitive to argument count
		return this.attr('class', ...arguments);
	}
	
	/**
	 * @see domAttrAsync
	 *
	 * @return Promise
	 */
	attr(requestOrPayload, valOrMutation = null, subValMutation = null) {
		// The ...arguments is important as the
		// domAttrAsync function is sensitive to argument count
		return domAttrAsync(this.__el(), ...arguments).then(ret => {
			return this.el && ret === this.el ? this : ret;
		});
	}
	
	/**
	 * @see domHtmlAsync
	 *
	 * @return Promise
	 */
	html(content) {
		// The ...arguments is important as the
		// domHtmlAsync function is sensitive to argument count
		return domHtmlAsync(this.__el(), ...arguments).then(ret => {
			return this.el && ret === this.el ? this : ret;
		});
	}
	
	/**
	 * @see domTextAsync
	 *
	 * @return Promise
	 */
	text(content) {
		// The ...arguments is important as the
		// domTextAsync function is sensitive to argument count
		return domTextAsync(this.__el(), ...arguments).then(ret => {
			return this.el && ret === this.el ? this : ret;
		});
	}
	
	/**
	 * @see domAppendAsync
	 *
	 * @return Promise
	 */
	append(content) {
		return domAppendAsync(this.__el(), ...arguments).then(() => {
			return this;
		});
	}
	
	/**
	 * @see domPrependAsync
	 *
	 * @return Promise
	 */
	prepend(content) {
		return domPrependAsync(this.__el(), ...arguments).then(() => {
			return this;
		});
	}
	
	/**
	 * @see cssComputeAsync
	 *
	 * @return Promise
	 */
	cssCompute(...args) {
		// The ...arguments is important as the
		// cssComputeAsync function is sensitive to argument count
		return cssComputeAsync(this.__el(), ...arguments);
	}
	
	/**
	 * Establishes a CSS operatiom that can be rolledback without altering similar operation by other code.
	 * If a callback is provided, it synces the entire operation with Reflow's normal read/write cycles.
	 *
	 * @param string|array			props
	 *
	 * @return Transaction
	 */
	cssTransaction(props) {
		return cssTransaction(this.__el(), props);
	}

	// ----------------------------------------
	
	/**
	 * @see on
	 */
	on(...args) {
		// The ...arguments is important as the
		// on function is sensitive to argument count
		return on(this.__el(), ...args);
	}
	
	/**
	 * @see off
	 */
	off(...args) {
		// The ...arguments is important as the
		// off function is sensitive to argument count
		return off(this.__el(), ...args);
	}
	
	/**
	 * @see trigger
	 */
	trigger(...args) {
		return trigger(this.__el(), ...args);
	}
	// ----------------------------------------
	
	/**
	 * Creates and plays an amiation.
	 * @see Animation
	 *
	 * @param array|object|string	effect
	 * @param object				params
	 *
	 * @return Promise
	 */
	play(effect, params = {}) {
		if (!('cancelForCss' in params)) {
			params.cancelForCss = true;
		}
		return (new Anim(this.__el(), effect, params)).play().then(() => this);
	}

	// ----------------------------------------
	
	/**
	 * Translates an element to a reference point in the UI.
	 *
	 * @param HTMLElement|Event|window|Chtml 	reference
	 * @param object							options
	 *
	 * @return Promise
	 */
	translateTo(reference, options = {}) {
		var refPoint = reference instanceof Chtml ? reference.el : reference;
		return Reflow.onread((resolve, reject) => {
			if (!this.el) {
				return reject();
			}
			var trans = uiTranslateTo(this.el, refPoint, options);
			if (options.play) {
				resolve(this.play([
					{transform: trans.current.from},
					{transform: trans.to},
				], options.play));
			} else {
				resolve(this.css({transform: trans.to}));
			}
		}, true/*withPromise*/);
	}
	
	/**
	 * Offsets an element to a reference point in the UI.
	 *
	 * @param HTMLElement|Event|window|Chtml 	reference
	 * @param object							options
	 *
	 * @return Promise
	 */
	offsetTo(reference, options = {}) {
		var refPoint = reference instanceof Chtml ? reference.el : reference;
		return Reflow.onread((resolve, reject) => {
			if (!this.el) {
				return reject();
			}
			var trans = uiOffsetTo(this.el, refPoint, options);
			if (options.play) {
				resolve(this.play([trans.current.from, trans.to], options.play));
			} else {
				resolve(this.css(trans.to));
			}
		}, true/*withPromise*/);
	}
	
	/**
	 * Scrolls an element to a reference point in the UI.
	 *
	 * @param HTMLElement|Event|window|Chtml 	reference
	 * @param object							options
	 *
	 * @return Promise
	 */
	scrollTo(reference, options = {}) {
		var refPoint = reference instanceof Chtml ? reference.el : reference;
		return Reflow.onread((resolve, reject) => {
			if (!this.el) {
				return reject();
			}
			var trans = uiScrollTo(this.el, refPoint, options);
			if (options.play) {
				console.log('scrollTo() does not currently support animation.');
			}
			resolve(trans.viewport.scrollTo(trans.to.scrollLeft, trans.to.scrollTop));
		}, true/*withPromise*/);
	}
};
