
/**
 * @imports
 */
import _isObject from '@web-native-js/commons/js/isObject.js';
import cssReadInline from './readInline.js';
import cssReadAsync from './readAsync.js';
import cssWriteAsync from './writeAsync.js';

/**
 * Applies some CSS within a transaction, gets computed values for use before rolling back.
 * If a callback is provided, it synces the entire operation with Reflow's normal read/write cycles.
 *
 * @param HTMLElement		el 
 * @param string|object		nameOrProps
 * @param string|number		val
 * @param function			readCallback
 *
 * @return Promise
 */
export default function(el, nameOrProps, val = null, readCallback = null) {
	var propsToRead = _isObject(nameOrProps) ? Object.keys(nameOrProps) : nameOrProps;
	readCallback = _isObject(nameOrProps) ? val : readCallback;
	// -------------
	var inlineSavepoint = cssReadInline(el, propsToRead);
	return cssWriteAsync(el, nameOrProps, val).then(() => {
		return cssReadAsync(el, readCallback || propsToRead).then(computedValues => {
			// We return the computedValues in a promise
			return cssWriteAsync(el, inlineSavepoint).then(() => {
				return computedValues;
			});
		});
	});
};
