
/**
 * @imports
 */
import _isString from '@web-native-js/commons/js/isString.js';
import _isArray from '@web-native-js/commons/js/isArray.js';
import _isObject from '@web-native-js/commons/js/isObject.js';
import _toCamel from '@web-native-js/commons/str/toCamel.js';
import _objFrom from '@web-native-js/commons/obj/from.js';
import _each from '@web-native-js/commons/obj/each.js';

/**
 * Gets or sets custom data.
 *
 * @param DOMElement			el 
 * @param string|array|object	requestOrPayload
 * @param mixed|void			val
 *
 * @return mixed
 */
export default function(el, requestOrPayload, val = null) {
	if (!el.dataset.__customDatasetKey) {
		el.dataset.__customDatasetKey = customDatasets.length;
		customDatasets[el.dataset.__customDatasetKey] = {};
	}
	var customDataset = customDatasets[el.dataset.__customDatasetKey];
	if (arguments.length === 2) {
		if (_isString(requestOrPayload)) {
			return customDataset[_toCamel(requestOrPayload)];
		}
		if (_isArray(requestOrPayload)) {
			var vals = {};
			requestOrPayload.forEach(key => {
				vals[key] = customDataset[_toCamel(key)];
			});
			return vals;
		}
	}
	var payload = requestOrPayload;
	if (!_isObject(requestOrPayload)) {
		payload = _objFrom(requestOrPayload, val);
	}
	_each(payload, (key, val) => {
		customDataset[_toCamel(key)] = val;
	});
};
const customDatasets = [];
