
/**
 * @imports
 */
import parsePlacement from './parsePlacement.js';
import evalPredicates from './evalPredicates.js';
import rect from './rect.js';

/**
 * Calculates the length on an axis of an intersection.
 *
 * @param HTMLElement|Event|window 	el
 * @param object		 			$rect
 * @param object		 			options
 *
 * @return object
 */
export default function(el, $rect, options = {}) {
	var $intersection = intersection(rect(options.using || el), $rect);
	var length = {x:'width', y:'height'};
	var start = {x:'left', y:'top'};
	var end = {x:'right', y:'bottom'};
	var $coords = {};
	var $alignment = {};
	['x', 'y'].forEach(axis => {
		if (options[axis] === false) {
			return;
		}
		var rect1Length = $intersection.rect1[length[axis]/*height*/];
		// Distinguish and predicate
		$alignment[axis] = parsePlacement(options[axis] || '');
		switch($alignment[axis].keyword) {
			case 'before':
				// Pull beyond start
				$coords[axis] = - ($intersection[start[axis]/*top*/] + rect1Length);
			break;
			case 'after':
				// Push beyond end
				$coords[axis] = $intersection[end[axis]/*bottom*/] + rect1Length;
			break;
			case 'start':
				// Pull to start
				$coords[axis] = - $intersection[start[axis]/*top*/];
			break;
			case 'end':
				// Push to end
				$coords[axis] = $intersection[end[axis]/*bottom*/];
			break;
			default:
				// Align to center
				$coords[axis] = $intersection.delta[axis];
		}
		// Apply predicates
		if ($alignment[axis].predicates) {
			$coords[axis] += evalPredicates($alignment[axis].predicates, rect1Length);
		}
	});
	$coords.alignment = $alignment;
	$coords.intersection = $intersection;
	return $coords;
};
