
/**
 * @imports
 */
import _intersect from '@web-native-js/commons/arr/intersect.js';
import _with from '@web-native-js/commons/obj/with.js';
import positioningOffsets from './positioningOffsets.js';
import coordsAtRect from './coordsAtRect.js';
import rect from './rect.js';

/**
 * Calculates an element's offset where it to be at the given rect.
 *
 * @param HTMLElement|Event|window 	el
 * @param HTMLElement|Event|window	el2
 * @param object 					options
 *
 * @return object
 */
export default function(el, el2, options = {}) {
	// -----------
	var $coords = coordsAtRect(el, rect(el2), options);
	// -----------
	var start = {x:'left', y:'top'};
	var end = {x:'right', y:'bottom'};
	var $offsets = _intersect(['x', 'y'], Object.keys($coords)).reduce((obj, axis) => {
		switch($coords.alignment[axis].keyword) {
			case 'before':
				return options.alternateAnchors ? _with(obj, start[axis], $coords[axis]) : _with(obj, end[axis], - $coords[axis]);
			case 'after':
				return options.alternateAnchors ? _with(obj, end[axis], - $coords[axis]) : _with(obj, start[axis], $coords[axis]);
			case 'end':
				return _with(obj, end[axis], - $coords[axis]);
			default: // center,start
				return _with(obj, start[axis], $coords[axis]);
		}
	}, {});
	// -----------
	var $offsetsFormatted = {from: {}, to: {}, current: {
			from: {}, to: {}, progress: {},
		},
	};
	// -----------
	var currentOffsets = positioningOffsets(el, Object.keys($offsets));
	['left', 'right', 'top', 'bottom'].forEach(name => {
		if (name in $offsets) {
			$offsetsFormatted.from[name] = 0;
			$offsetsFormatted.to[name] = $offsets[name] + currentOffsets[name];
			// ----------------
			$offsetsFormatted.current.from[name] = currentOffsets[name];
			$offsetsFormatted.current.to[name] = $offsets[name];
			// ----------------
			$offsetsFormatted.current.progress[name] = Math.abs(currentOffsets[name] / ($offsets[name] + currentOffsets[name]));
		} else {
			$offsetsFormatted.from[name] = 'auto';
			$offsetsFormatted.to[name] = 'auto';
			// ----------------
			$offsetsFormatted.current.from[name] = 'auto';
			$offsetsFormatted.current.to[name] = 'auto';
		}
	});
	return $offsetsFormatted;
};
