
/**
 * @imports
 */
import _isNumeric from '@web-native-js/commons/js/isNumeric.js';
import cssRead from '../Css/read.js';
import rect from './rect.js';

/**
 * Simulates, does not actually effect, a DOMElement to a some CSS position and
 * uses rect() to calculate its rects at that virtual position.
 *
 * @param DOMElement				el 
 * @param object 					offsets
 *
 * @return object
 */
export default function(el, offsets) {
	var $rect = rect(el);
	var positionType = cssRead(el, 'position');
	if (positionType !== 'static') {
		if (positionType === 'absolute') {
			var offsetParentRect = rect(el.offsetParent);
			$rect.top = offsetParentRect.top;
			$rect.left = offsetParentRect.left;
		} else if (positionType === 'fixed') {
			var offsetParentRect = rect(window);
			$rect.top = 0;
			$rect.left = 0;
		}
		$rect.top = _isNumeric(offsets.top) ? $rect.top + parseFloat(offsets.top) 
			: (_isNumeric(offsets.bottom) ? (offsetParentRect 
				? (offsetParentRect.top + offsetParentRect.height) - ($rect.height + offsets.bottom)
				: $rect.top - offsets.bottom
			) : 0);
		$rect.left = _isNumeric(offsets.left) ? $rect.left + parseFloat(offsets.left) 
			: (_isNumeric(offsets.right) ? (offsetParentRect 
				? (offsetParentRect.left + offsetParentRect.width) - ($rect.width + offsets.right)
				: $rect.left - offsets.right
			) : 0);
	}
	return $rect;
};
