
/**
 * @imports
 */
import _isObject from '@web-native-js/commons/js/isObject.js';
import _copyPlain from '@web-native-js/commons/obj/copyPlain.js';

/**
 * Observes when any of the given elements
 * changes in size.
 *
 * @param [HTMLElement]			 	els
 * @param function					callback
 *
 * @return ResizeObserver
 */
export default function(els, callback) {
	var sizings = Object.keys(els);
	var ro = new ResizeObserver(entries => {
		for (let entry of entries) {
			sizings[els.indexOf(entry.target)] = {
				width: entry.contentRect.left + entry.contentRect.right, 
				height: entry.contentRect.top + entry.contentRect.bottom,
			};
		}
		sizings = sizings.map((sizing, i) => !_isObject(sizing) ? _copyPlain(rect(els[i]), ['width', 'height',]) : sizing);
		callback.call(...sizings);
	});
	els.forEach(el => {
		ro.observe(el);
	});
	return ro;
};
