
/**
 * @imports
 */
import scrolls from './scrolls.js';
import scrollParent from './scrollParent.js';
import coordsAtRect from './coordsAtRect.js';
import rect from './rect.js';

/**
 * Scrolls an element to the position of another element
 * within its scrollable parent.
 *
 * @param HTMLElement|Event|window 	el
 * @param HTMLElement|Event|window	el2
 * @param object 					options
 *
 * @return object
 */
export default function(el, el2, options = {}) {
	var viewport = options.scrollAnchor || scrollParent(el);
	viewport = viewport === document.body ? window : viewport;
	if (!scrolls(viewport)) {
		return;
	}
	// -----------
	var $coords = coordsAtRect(el, rect(el2), options);
	// -----------
	var $offsetsFormatted = {from: {}, to: {}, current: {
			from: {}, to: {}, progress: {},
		},
		viewport: viewport,
	};
	// -----------
	var currentScroll = {
		scrollLeft: viewport[viewport === window ? 'pageXOffset' : 'scrollLeft'],
		scrollTop: viewport[viewport === window ? 'pageYOffset' : 'scrollTop'],
	};
	['x', 'y'].forEach((axis, i) => {
		var axisProp = axis === 'x' ? 'scrollLeft' : 'scrollTop';
		if (axis in $coords) {
			$coordsFormatted.from[axisProp] = 0;
			$coordsFormatted.to[axisProp] = $coords[axis] + currentScroll[axisProp];
			// ----------------
			$coordsFormatted.current.from[axisProp] = currentScroll[axisProp];
			$coordsFormatted.current.to[axisProp] = $coords[axis];
			// ----------------
			$coordsFormatted.current.progress[axis] = Math.abs(currentScroll[axisProp] / ($coords[axis] + currentScroll[axisProp]));
		} else {
			$coordsFormatted.from[axisProp] = currentScroll[axisProp];
			$coordsFormatted.to[axisProp] = currentScroll[axisProp];
			// ----------------
			$coordsFormatted.current.from[axisProp] = currentScroll[axisProp];
			$coordsFormatted.current.to[axisProp] = currentScroll[axisProp];
			// ----------------
			$coordsFormatted.current.progress[axis] = 0;
		}
	});
	return $coordsFormatted;
};
