
/**
 * @imports
 */
import cssRead from '../Css/read.js';
import TransformRule from '../Css/TransformRule.js';
import coordsAtRect from './coordsAtRect.js';
import rect from './rect.js';

/**
 * Manipulates an element's translate.translate to place it with another element.
 *
 * @param HTMLElement|Event|window 	el
 * @param HTMLElement|Event|window	el2
 * @param object 					options
 *
 * @return object
 */
export default function(el, el2, options = {}) {
	// -----------
	var $coords = coordsAtRect(el, rect(el2), options);
	// -----------
	var $coordsFormatted = {
		from: new TransformRule({translate: [],}),
		to: new TransformRule({translate: [],}),
		current: {
			from: new TransformRule({translate: [],}),
			to: new TransformRule({translate: [],}),
			progress: {},
		},
	};
	// -----------
	var activeTransform = cssRead(el, 'transform');
	['x', 'y'].forEach((axis, i) => {
		if (axis in $coords) {
			$coordsFormatted.from.translate[i] = 0;
			$coordsFormatted.to.translate[i] = $coords[axis] + activeTransform.translate[i];
			// ----------------
			$coordsFormatted.current.from.translate[i] = activeTransform.translate[i];
			$coordsFormatted.current.to.translate[i] = $coords[axis];
			// ----------------
			$coordsFormatted.current.progress[axis] = Math.abs(activeTransform.translate[i] / ($coords[axis] + activeTransform.translate[i]));
		} else {
			$coordsFormatted.from.translate[i] = activeTransform.translate[i];
			$coordsFormatted.to.translate[i] = activeTransform.translate[i];
			// ----------------
			$coordsFormatted.current.from.translate[i] = activeTransform.translate[i];
			$coordsFormatted.current.to.translate[i] = activeTransform.translate[i];
			// ----------------
			$coordsFormatted.current.progress[axis] = 0;
		}
	});
	return $coordsFormatted;
};
